/*
 * $Id: selfcertificate.cpp,v 1.3 2007/04/02 10:44:53 ozawa Exp $
 *
 * Copyright 2005- ONGS Inc. All rights reserved.
 * 
 * author: Masanori OZAWA (ozawa@ongs.co.jp)
 * version: $Revision: 1.3 $
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY ONGS INC ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL ONGS INC OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * The views and conclusions contained in the software and documentation are
 * those of the authors and should not be interpreted as representing official
 * policies, either expressed or implied, of the ONGS Inc.
 * 
 */

#include "../include/fusenshi.h"
#include "selfcertificate.h"

#include "gnutls/x509.h"
#include <time.h>

#define GNUTLS_CHECK(method)	{ \
	ret = method; \
	if (ret < 0) { \
		fprintf(stderr, "%s: Unable to create self-signed certificate. (%s)\n", \
		    APP_NAME, gnutls_strerror(ret)); \
		goto mk_failed; \
	}\
}

gnutls_certificate_credentials_t SelfCertificate::m_cert = NULL;

/**
 * ʾޤ
 * 䵻椬ưLANĶϤʤꤹޤǤʤ
 * ޤưפȽ̲ǽǤ뤳ȤȤƤޤ
 * 
 * @return  0 ᤷޤ
 */
int makeSelfCertificate(gnutls_certificate_credentials_t& cert)
{
	gnutls_x509_crt_t crt, cacrt;
	gnutls_x509_privkey_t key, cakey;
	uint32_t serial;
	unsigned char keyid[128];
	size_t keyidlen = sizeof(keyid);
	int ret = 0;

	serial = (uint32_t)time(NULL);

	crt = cacrt = NULL;
	key = cakey = NULL;
    
    GNUTLS_CHECK(gnutls_x509_privkey_init(&key));
    GNUTLS_CHECK(gnutls_x509_privkey_generate(key, GNUTLS_PK_RSA, SSL_DH_BITS, 0));
    GNUTLS_CHECK(gnutls_x509_privkey_init(&cakey));
    GNUTLS_CHECK(gnutls_x509_privkey_generate(cakey, GNUTLS_PK_RSA, SSL_DH_BITS, 0));
    
    GNUTLS_CHECK(gnutls_x509_crt_init(&cacrt));
	GNUTLS_CHECK(gnutls_x509_crt_set_dn_by_oid(cacrt, 
		GNUTLS_OID_X520_ORGANIZATION_NAME, 0, "ONGS Inc.", strlen("ONGS Inc.")));
	GNUTLS_CHECK(gnutls_x509_crt_set_dn_by_oid(cacrt, 
		GNUTLS_OID_X520_ORGANIZATIONAL_UNIT_NAME, 0,
		APP_NAME ": auto-generated self-signed certificate",
		strlen(APP_NAME ": auto-generated self-signed certificate")));
	GNUTLS_CHECK(gnutls_x509_crt_set_dn_by_oid(cacrt,
		GNUTLS_OID_X520_COMMON_NAME, 0, APP_NAME, strlen(APP_NAME)));

	GNUTLS_CHECK(gnutls_x509_crt_set_key(cacrt, cakey));
	GNUTLS_CHECK(gnutls_x509_crt_set_serial(cacrt, &serial, sizeof(serial)));
	GNUTLS_CHECK(gnutls_x509_crt_set_activation_time(cacrt, time(NULL)));
	GNUTLS_CHECK(gnutls_x509_crt_set_expiration_time(cacrt, 0xffffffff)); /* ʵ */
	GNUTLS_CHECK(gnutls_x509_crt_set_ca_status(cacrt, 0));
	GNUTLS_CHECK(gnutls_x509_crt_set_version(cacrt, 3));
	GNUTLS_CHECK(gnutls_x509_crt_get_key_id(cacrt, 0, keyid, &keyidlen));
	GNUTLS_CHECK(gnutls_x509_crt_set_subject_key_id(cacrt, keyid, keyidlen));
	GNUTLS_CHECK(gnutls_x509_crt_sign(cacrt, cacrt, cakey));

	GNUTLS_CHECK(gnutls_x509_crt_init(&crt));
	GNUTLS_CHECK(gnutls_x509_crt_set_dn_by_oid(crt, 
		GNUTLS_OID_X520_ORGANIZATION_NAME, 0, "ONGS Inc.", strlen("ONGS Inc.")));
	GNUTLS_CHECK(gnutls_x509_crt_set_dn_by_oid(crt, 
		GNUTLS_OID_X520_ORGANIZATIONAL_UNIT_NAME, 0,
		APP_NAME ": auto-generated self-signed certificate",
		strlen(APP_NAME ": auto-generated self-signed certificate")));
	GNUTLS_CHECK(gnutls_x509_crt_set_dn_by_oid(crt,
		GNUTLS_OID_X520_COMMON_NAME, 0, APP_NAME, strlen(APP_NAME)));

	GNUTLS_CHECK(gnutls_x509_crt_set_key(crt, key));
	GNUTLS_CHECK(gnutls_x509_crt_set_serial(crt, &serial, sizeof(serial)));
	GNUTLS_CHECK(gnutls_x509_crt_set_activation_time(crt, time(NULL)));
	GNUTLS_CHECK(gnutls_x509_crt_set_expiration_time(crt, 0xffffffff)); /* ʵ */
	GNUTLS_CHECK(gnutls_x509_crt_set_ca_status(crt, 0));
	GNUTLS_CHECK(gnutls_x509_crt_set_version(crt, 3));
	keyidlen = sizeof(keyid);
	GNUTLS_CHECK(gnutls_x509_crt_get_key_id(crt, 0, keyid, &keyidlen));
	GNUTLS_CHECK(gnutls_x509_crt_set_subject_key_id(crt, keyid, keyidlen));
	GNUTLS_CHECK(gnutls_x509_crt_sign(crt, crt, key));

	GNUTLS_CHECK(gnutls_certificate_allocate_credentials(&cert));
	GNUTLS_CHECK(gnutls_certificate_set_x509_trust(cert, &cacrt, 1));
	GNUTLS_CHECK(gnutls_certificate_set_x509_key(cert, &crt, 1, key));

mk_failed:
	if (key) gnutls_x509_privkey_deinit(key);
	if (cakey) gnutls_x509_privkey_deinit(cakey);
	if (cacrt) gnutls_x509_crt_deinit(cacrt);
	if (crt) gnutls_x509_crt_deinit(crt);
	
	return ret;
}


SelfCertificate::SelfCertificate()
{
}

SelfCertificate::~SelfCertificate()
{
	destorySelfCertificate();
}

gnutls_certificate_credentials_t SelfCertificate::getSelfCertificate()
{
    if (!m_cert)
        makeSelfCertificate(m_cert);
    
    return m_cert;
}

void SelfCertificate::destorySelfCertificate()
{
	if (m_cert) {
		gnutls_certificate_free_credentials(m_cert);
		m_cert = NULL;
	}
}
