/*
 * $Id: fusenshisslserversocket.cpp,v 1.2 2007/04/02 10:44:52 ozawa Exp $
 *
 * Copyright 2005- ONGS Inc. All rights reserved.
 * 
 * author: Masanori OZAWA (ozawa@ongs.co.jp)
 * version: $Revision: 1.2 $
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY ONGS INC ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL ONGS INC OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * The views and conclusions contained in the software and documentation are
 * those of the authors and should not be interpreted as representing official
 * policies, either expressed or implied, of the ONGS Inc.
 * 
 */

#include "../include/fusenshi.h"
#include "fusenshisslserversocket.h"

#include "../crypto/selfcertificate.h"

//_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/
class FusenshiSSLServerConnectedSocket : public FusenshiSocket
{
public:
    FusenshiSSLServerConnectedSocket()
    {
        m_session = NULL;
    }
    

    virtual ~FusenshiSSLServerConnectedSocket()
    {
        close();
    }

    bool sslAccept(int sock)
    {
    	int ret;
		const int cert_priority[] = {
			GNUTLS_CRT_X509, 0
		};

        m_socket = sock;

        // SSL
        // ̿Ź沽ޤۥȤξϹԤޤ
        
        gnutls_certificate_credentials_t cert = SelfCertificate::getSelfCertificate();
        if (!cert) return false;

		gnutls_init(&m_session, GNUTLS_SERVER);
		gnutls_set_default_export_priority(m_session);
		gnutls_certificate_type_set_priority(m_session, cert_priority);

		gnutls_credentials_set(m_session, GNUTLS_CRD_CERTIFICATE, cert);
		gnutls_certificate_server_set_request(m_session, GNUTLS_CERT_REQUIRE);
		gnutls_dh_set_prime_bits(m_session, SSL_DH_BITS);

		gnutls_transport_set_ptr(m_session, (gnutls_transport_ptr_t)sock);
		ret = gnutls_handshake(m_session);
		if (ret < 0) {
			gnutls_deinit(m_session);
			m_session = NULL;
			fprintf(stderr, "%s: SSL/TLS handshake has failed. (%s)\n", APP_NAME, gnutls_strerror(ret));
            return false;
        }

        return true;
    }

    virtual ssize_t recv(void* buf, size_t len)
    {
        return gnutls_record_recv(m_session, buf, len);
    }
    
    virtual ssize_t send(const void* buf, size_t len)
    {
        return gnutls_record_send(m_session, buf, len);
    }

    virtual int shutdown()
    {
        if (m_session)
        	gnutls_bye(m_session, GNUTLS_SHUT_RDWR);
        else
            return FusenshiSocket::shutdown();
        
        return 0;
    }
    
    virtual int close()
    {
        shutdown();
        
        int result = FusenshiSocket::close();
    
        if (m_session) {
            gnutls_deinit(m_session);
            m_session = NULL;
        }

        return result;
    }
    
    virtual bool operator()()
    {
        return (NULL != m_session);
    }
    
private:
    gnutls_session_t m_session;
};

//_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/
FusenshiSSLServerSocket::FusenshiSSLServerSocket(int port) :
    FusenshiServerSocket(port)
{
}

FusenshiSSLServerSocket::~FusenshiSSLServerSocket()
{
}

RefPtr<FusenshiSocket> FusenshiSSLServerSocket::accept
    (struct sockaddr * addr, socklen_t * addrlen)
{
    int cs = ::accept(m_socket, addr, addrlen);
    
    if (0 <= cs) {
        FusenshiSSLServerConnectedSocket *csock = new FusenshiSSLServerConnectedSocket();
        if (csock->sslAccept(cs)) {
            return RefPtr<FusenshiSocket>(csock);
        }
        delete csock;
    }
    
    return RefPtr<FusenshiSocket>();
}
